/**
 *   common-jbi-runtime.jar - Set of classes used in building JBI Components
 *
 *   Copyright (C) 2007 Gestalt, LLC. All Rights Reserved.
 *   http://www.gestalt-llc.com/
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License version 2.1 as published by the Free Software Foundation.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
 */

package com.gestalt.jbi.component.bootstrap;

import com.sun.jbi.internationalization.Messages;

import javax.jbi.JBIException;
import javax.jbi.component.Bootstrap;
import javax.jbi.component.InstallationContext;
import javax.jbi.component.ComponentContext;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * @author rriven
 *
 */
public abstract class AbstractBootstrapper implements Bootstrap {

	protected Logger log = Messages.getLogger(AbstractBootstrapper.class);
	private Messages messages = Messages.getMessages(AbstractBootstrapper.class);

    protected InstallationContext installationContext;
	private ObjectName mBeanName;
	private MBeanServer mBeanServer;
	
	/* (non-Javadoc)
	 * @see javax.jbi.component.Bootstrap#cleanUp()
	 */
	public void cleanUp() throws JBIException {
        if (log.isLoggable(Level.FINE)) {
            log.log(Level.FINE, "Bootstrap: Cleaing Up");
        }
        doCleanUp();
		
		if (mBeanName == null) {
			return ;
		}
		if (mBeanServer == null) {
            log.log(Level.WARNING, messages.getString("CJBIRT-R00100.nullMbean"));
            throw new JBIException("A null MBeanServer was detected.");
		}
		try {
			if (mBeanServer.isRegistered(mBeanName)) {
				mBeanServer.unregisterMBean(mBeanName);
			}
		} catch (Exception e) {
            log.log(Level.WARNING,messages.getString("CJBIRT-W00105.bootstrapCleanupException"));
            throw new JBIException("Error managing the bootstrapper MBean: " + e.getMessage(), e);
		}
	}
	
	public abstract void doCleanUp() throws JBIException;

	/* (non-Javadoc)
	 * @see javax.jbi.component.Bootstrap#getExtensionMBeanName()
	 */
	public ObjectName getExtensionMBeanName() {
		return mBeanName;
	}

	/* (non-Javadoc)
	 * @see javax.jbi.component.Bootstrap#init(javax.jbi.component.InstallationContext)
	 */
	public void init(InstallationContext installationContext) throws JBIException {
		if (installationContext == null) {
            log.log(Level.WARNING,messages.getString("CJBIRT-W00110.bootstrapInitNullInstallationContext"));
            throw new JBIException("A null InstallationContext was detected.");
		}

        ComponentContext ctx = installationContext.getContext();
        Messages.registerContext(ctx);

        log.log(Level.INFO, messages.getString("CJBIRT-R00115.bootstrapInitializing"));

        this.installationContext = installationContext;
		mBeanName = installationContext.getContext().getMBeanNames().createCustomComponentMBeanName("bootstrapper");
		mBeanServer = installationContext.getContext().getMBeanServer();			
		
		if (mBeanServer == null) {
            log.log(Level.WARNING,messages.getString("CJBIRT-W00120.nullMbean"));
            throw new JBIException("A null MBeanServer was detected.");
		}
		
		doInit();
			
		try {
			if (mBeanServer.isRegistered(mBeanName)) {
				mBeanServer.unregisterMBean(mBeanName);
			}
			Object mBean = createMBean();
			if (mBean != null) {
				mBeanServer.registerMBean(createMBean(), mBeanName);
			}
		} catch (Exception e) {
            log.log(Level.WARNING,messages.getString("CJBIRT-W00125.bootstrapInitNullInstallationContext"));
            throw new JBIException("Error managing bootstrapper MBean: " + e.getMessage(), e);
		}

        log.log(Level.INFO, messages.getString("CJBIRT-R00130.bootstrapInitialized"));
    }
	
	public abstract Object createMBean();
	
	public abstract void doInit() throws JBIException;	

	/* (non-Javadoc)
	 * @see javax.jbi.component.Bootstrap#onInstall()
	 */
	public void onInstall() throws JBIException {
		log.log(Level.INFO, messages.getString("CJBIRT-R00135.bootstrapInstalling"));
		doInstall();
        log.log(Level.INFO, messages.getString("CJBIRT-R00140.bootstrapInstalled"));
    }
	
	public abstract void doInstall() throws JBIException;

	/* (non-Javadoc)
	 * @see javax.jbi.component.Bootstrap#onUninstall()
	 */
	public void onUninstall() throws JBIException {
		log.log(Level.INFO, messages.getString("CJBIRT-R00145.bootstrapUninstalling"));
		doUninstall();
        log.log(Level.INFO, messages.getString("CJBIRT-R00150.bootstrapUninstalled"));
    }
	
	public abstract void doUninstall() throws JBIException;

}
