/**
 *   common-jbi-runtime.jar - Set of classes used in building JBI Components
 *
 *   Copyright (C) 2007 Gestalt, LLC. All Rights Reserved.
 *   http://www.gestalt-llc.com
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License version 2.1 as published by the Free Software Foundation.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
 */
package com.gestalt.jbi.component.descriptor;

import com.sun.jbi.internationalization.Messages;

import org.xmlpull.mxp1.MXParser;

import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.namespace.QName;


public class JBIDescriptorParser {
    private static final String FOLDER = "META-INF";
    private static final String FILE_NAME = "jbi.xml";
    private Logger log = Messages.getLogger(JBIDescriptorParser.class);
    private Messages messages = Messages.getMessages(JBIDescriptorParser.class);
    private Boolean bindingComponent;
    private List<Provider> providers = new ArrayList<Provider>();
    private List<Consumer> consumers = new ArrayList<Consumer>();

    public JBIDescriptorParser(String serviceUnitRootPath) {
        if (log.isLoggable(Level.FINE)) {
            log.log(Level.FINE,
                serviceUnitRootPath + File.separator + FOLDER + File.separator +
                FILE_NAME);
        }

        parseDescriptor(new File(serviceUnitRootPath + File.separator + FOLDER,
                FILE_NAME));
    }

    private void parseDescriptor(File descriptor) {
        try {
            XmlPullParser xpp = new MXParser();
            xpp.setFeature(XmlPullParser.FEATURE_PROCESS_NAMESPACES, true);
            xpp.setInput(new FileReader(descriptor));
            parseDescriptor(xpp);
        } catch (Exception e) {
            log.log(Level.WARNING,
                messages.getString(
                    "CJBIRT-W00200.exceptionParsingJBIDescriptor"), e);
        }
    }

    private void parseDescriptor(XmlPullParser xpp)
        throws XmlPullParserException, IOException {
        boolean done = false;

        while (!done) {
            int eventType = xpp.next();

            if (eventType == XmlPullParser.START_TAG) {
                if ("services".equals(xpp.getName())) {
                    bindingComponent = Boolean.valueOf(xpp.getAttributeValue(
                                null, "binding-component"));
                } else if ("provides".equals(xpp.getName())) {
                    if (log.isLoggable(Level.FINE)) {
                        log.log(Level.FINE,
                            "A 'provides' endpoint description was detected");
                    }

                    String interfaceName = xpp.getAttributeValue(null,
                            "interface-name");
                    String serviceName = xpp.getAttributeValue(null,
                            "service-name");
                    String endpointName = xpp.getAttributeValue(null,
                            "endpoint-name");
                    QName finalInterfaceName = null;
                    if (interfaceName != null) {
                        finalInterfaceName = resolveQName(interfaceName, xpp);
                    }
                    QName finalServiceName = resolveQName(serviceName, xpp);

                    addProvider(finalServiceName, finalInterfaceName,
                        endpointName);
                } else if ("consumes".equals(xpp.getName())) {
                    if (log.isLoggable(Level.FINE)) {
                        log.log(Level.FINE,
                            "A 'consumes' endpoint description was detected");
                    }

                    String interfaceName = xpp.getAttributeValue(null,
                            "interface-name");
                    String serviceName = xpp.getAttributeValue(null,
                            "service-name");
                    String endpointName = xpp.getAttributeValue(null,
                            "endpoint-name");
                    String linkType = xpp.getAttributeValue(null, "link-type");

                    QName finalInterfaceName = resolveQName(interfaceName, xpp);
                    QName finalServiceName = resolveQName(serviceName, xpp);

                    addConsumer(finalServiceName, finalInterfaceName,
                        endpointName, linkType);
                }
            } else if (eventType == XmlPullParser.END_DOCUMENT) {
                done = true;
            }
        }
    }

    public Boolean isBindingComponent() {
        return bindingComponent;
    }

    public List<Consumer> getConsumers() {
        return consumers;
    }

    public List<Provider> getProviders() {
        return providers;
    }

    private void addProvider(QName serviceName, QName interfaceName,
        String endpointName) {
        log.log(Level.INFO,
            messages.getString("CJBIRT-R00205.addingProvider",
                serviceName + " : " + interfaceName + " : " + endpointName));
        providers.add(new Provider(serviceName, interfaceName, endpointName));
    }

    private void addConsumer(QName serviceName, QName interfaceName,
        String endpointName, String linkType) {
        log.log(Level.INFO,
            messages.getString("CJBIRT-R00210.addingConsumer",
                serviceName + " : " + interfaceName + " : " + endpointName +
                " : " + linkType));
        consumers.add(new Consumer(serviceName, interfaceName, endpointName,
                linkType));
    }

    private QName resolveQName(String value, XmlPullParser xpp) {
        String[] parts = value.split(":");

        if (parts.length > 1) {
            return new QName(xpp.getNamespace(parts[0]), parts[1], parts[0]);
        }

        return new QName(null, value, null);
    }

    public class Consumer {
        private QName serviceName;
        private QName interfaceName;
        private String endpointName;
        private String linkType;

        public Consumer(QName serviceName, QName interfaceName,
            String endpointName, String linkType) {
            this.serviceName = serviceName;
            this.interfaceName = interfaceName;
            this.endpointName = endpointName;
            this.linkType = linkType;
        }

        public String getEndpointName() {
            return endpointName;
        }

        public QName getInterfaceName() {
            return interfaceName;
        }

        public String getLinkType() {
            return linkType;
        }

        public QName getServiceName() {
            return serviceName;
        }
    }

    public class Provider {
        private QName serviceName;
        private QName interfaceName;
        private String endpointName;

        public Provider(QName serviceName, QName interfaceName,
            String endpointName) {
            this.serviceName = serviceName;
            this.interfaceName = interfaceName;
            this.endpointName = endpointName;
        }

        public String getEndpointName() {
            return endpointName;
        }

        public QName getInterfaceName() {
            return interfaceName;
        }

        public QName getServiceName() {
            return serviceName;
        }
    }
}
