/**
 *   common-jbi-runtime.jar - Set of classes used in building JBI Components
 *
 *   Copyright (C) 2007 Gestalt, LLC. All Rights Reserved.
 *   http://www.gestalt-llc.com/
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License version 2.1 as published by the Free Software Foundation.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
 */

package com.gestalt.jbi.component.lifecycle;

import com.gestalt.jbi.component.AbstractComponent;
import com.sun.jbi.internationalization.Messages;

import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.component.ComponentLifeCycle;
import javax.jbi.messaging.DeliveryChannel;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * @author rriven
 *
 */
public abstract class AbstractComponentLifeCycle implements ComponentLifeCycle {

	protected Logger log = Messages.getLogger(AbstractComponentLifeCycle.class);;
    private Messages messages = Messages.getMessages(AbstractComponentLifeCycle.class);

    protected AbstractComponent component;
	
	protected ComponentContext componentContext;
	
	protected DeliveryChannel deliveryChannel;
	
	protected ObjectName mBeanName;
	
	protected MBeanServer mBeanServer;
	
	protected Object mBean;
	
	protected AbstractMessageExchangeProcessor processor;
	
	public AbstractComponentLifeCycle(AbstractComponent component) {
		this.component = component;		
	}
	
	/* (non-Javadoc)
	 * @see javax.jbi.component.ComponentLifeCycle#getExtensionMBeanName()
	 */
	public ObjectName getExtensionMBeanName() {
		return mBeanName;
	}
	
	public ComponentContext getComponentContext() {
		return componentContext;
	}

	/* (non-Javadoc)
	 * @see javax.jbi.component.ComponentLifeCycle#init(javax.jbi.component.ComponentContext)
	 */
	public void init(ComponentContext componentContext) throws JBIException {
        Messages.registerContext(componentContext);
        log = Messages.getLogger(AbstractComponentLifeCycle.class);

        log.log(Level.INFO, messages.getString("CJBIRT-R00700.componentLifecycleInitializing"));
        
        this.componentContext = componentContext;
		mBeanName = createMBeanName();
		mBeanServer = componentContext.getMBeanServer();
		processor = createMessageExchangeProcessor();
		processor.init();
		doInit();

        if (mBeanName == null) {
            log.log(Level.WARNING,messages.getString("CJBIRT-W00705.unableToCreateMBeanNameDuringComponentLifecycleInitialization"));
		}
		if (mBeanServer == null) {
            log.log(Level.WARNING,messages.getString("CJBIRT-W00710.unableToRetrieveMBeanServerDuringComponentLifecycleInitialization"));
		}

        try {
            mBean = createMBean();
            if (mBean == null) {
                log.log(Level.WARNING,messages.getString("CJBIRT-W00715.unableToCreateMBeanDuringComponentLifecycleInitialization"));
                return;
            }
			if (mBeanServer.isRegistered(mBeanName)) {
				mBeanServer.unregisterMBean(mBeanName);
			}
			mBeanServer.registerMBean(mBean, mBeanName);
		} catch (Exception e) {
			throw new JBIException("Error managing the lifecycle MBean: " + e.getMessage(), e);
		}
        log.log(Level.INFO, messages.getString("CJBIRT-R00720.componentLifecycleInitialized"));
    }
	
	public abstract Object createMBean();
	
	public abstract ObjectName createMBeanName();
	
	public abstract AbstractMessageExchangeProcessor createMessageExchangeProcessor();
	
	public abstract void doInit() throws JBIException;

	/* (non-Javadoc)
	 * @see javax.jbi.component.ComponentLifeCycle#shutDown()
	 */
	public void shutDown() throws JBIException {
		log.log(Level.INFO, messages.getString("CJBIRT-R00725.componentLifecycleShuttingDown"));
		doShutDown();
		processor.shutdown();
		if (mBeanName == null) {
			return ;
		}
		if (mBeanServer == null) {
                log.log(Level.WARNING,messages.getString("CJBIRT-W00730.nullMBeanServerWasDetected"));
            return ;
		}
		try {
			if (mBeanServer.isRegistered(mBeanName)) {
				mBeanServer.unregisterMBean(mBeanName);
			}
		} catch (Exception e) {
            log.log(Level.WARNING,messages.getString("CJBIRT-W00735.unableToManageLifecycleMBean"));
            throw new JBIException("Error managing the lifecycle MBean: " + e.getMessage(), e);
		}
        log.log(Level.INFO,messages.getString("CJBIRT-R00740.componentLifecycleShutDown"));
    }
	
	public abstract void doShutDown() throws JBIException;

	/* (non-Javadoc)
	 * @see javax.jbi.component.ComponentLifeCycle#start()
	 */
	public void start() throws JBIException {
		log.log(Level.INFO, messages.getString("CJBIRT-R00745.componentLifecycleStarting"));
		deliveryChannel = componentContext.getDeliveryChannel();
		processor.start();
		doStart();
        log.log(Level.INFO,messages.getString("CJBIRT-R00740.componentLifecycleStarted"));
    }
	
	public abstract void doStart() throws JBIException;

	/* (non-Javadoc)
	 * @see javax.jbi.component.ComponentLifeCycle#stop()
	 */
	public void stop() throws JBIException {
		log.log(Level.INFO, messages.getString("CJBIRT-R00745.componentLifecycleStopping"));
		doStop();
		processor.stop();
        log.log(Level.INFO,messages.getString("CJBIRT-R00750.componentLifecycleStopped"));
    }
	
	public abstract void doStop() throws JBIException;

	public DeliveryChannel getDeliveryChannel() {
		return deliveryChannel;
	}

}
