/**
 *   common-jbi-runtime.jar - Set of classes used in building JBI Components
 *
 *   Copyright (C) 2007 Gestalt, LLC. All Rights Reserved.
 *   http://www.gestalt-llc.com/
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License version 2.1 as published by the Free Software Foundation.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
 */

package com.gestalt.jbi.component.lifecycle;

import com.gestalt.jbi.component.AbstractComponent;
import com.gestalt.jbi.component.handler.AbstractMessageExchangeHandler;
import com.gestalt.jbi.component.manager.AbstractServiceUnitManager;
import com.gestalt.jbi.component.manager.Endpoint;
import com.sun.jbi.internationalization.Messages;

import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;
import java.util.logging.Level;
import java.util.logging.Logger;

public class AbstractMessageExchangeProcessor {

    protected static final Logger log = Messages.getLogger(AbstractMessageExchangeProcessor.class);
    private Messages messages = Messages.getMessages(AbstractMessageExchangeProcessor.class);

    private volatile Boolean canContinue = true;
    private volatile Boolean canAccept = false;

    private AbstractComponent component;
    private AbstractComponentLifeCycle componentLifeCycle;
    private AbstractServiceUnitManager serviceUnitManager;

    private ExecutorService processorThreadPool;
    private ExecutorService handlerThreadPool;

    private Lock acceptMutex = new ReentrantLock();
    private Lock lifeMutex = new ReentrantLock();

    public AbstractMessageExchangeProcessor(AbstractComponent component) {
        this.component = component;
        this.componentLifeCycle = (AbstractComponentLifeCycle) component.getLifeCycle();
        this.serviceUnitManager = (AbstractServiceUnitManager) component.getServiceUnitManager();

    }

    public void init() {
        handlerThreadPool = Executors.newFixedThreadPool(5);
        processorThreadPool = Executors.newSingleThreadExecutor();
        processorThreadPool.execute(new Runnable() {
            public void run() {
                while (canContinue) {
                    if (canAccept) {
                        processExchange();
                    } else {
                        try {
                            Thread.sleep(2000);
                        } catch (InterruptedException e) {
                            if (log.isLoggable(Level.FINEST)) {
                                log.log(Level.FINEST,"An InterruptedException was received while sleeping.  No action is required.",e);
                            }
                        }
                    }
                }
            }
        });
    }

    public void shutdown() {
        lifeMutex.lock();
        canContinue = false;
        lifeMutex.unlock();
    }

    public void start() {
        acceptMutex.lock();
        canAccept = true;
        acceptMutex.unlock();
    }

    public void stop() {
        acceptMutex.lock();
        canAccept = false;
        acceptMutex.unlock();
    }

    protected void processExchange() {
        DeliveryChannel channel = componentLifeCycle.getDeliveryChannel();
        if (channel == null) {
            return;
        }
        try {
            MessageExchange exchange = channel.accept(3000);
            if (exchange == null) {
                if (log.isLoggable(Level.FINEST)) {
                    log.log(Level.FINEST, "The MessageExchange is Null");
                }
                return;
            }
            ServiceEndpoint serviceEndpoint = exchange.getEndpoint();
            if (log.isLoggable(Level.FINE)) {
                log.log(Level.FINE,"Received a message for the endpoint: " + serviceEndpoint);
            }
            AbstractMessageExchangeHandler handler = findHandler(serviceEndpoint.getServiceName(), serviceEndpoint.getEndpointName(),exchange.getRole());
            if (handler == null) {
                log.log(Level.WARNING, messages.getString("CJBIRT-W00760.noHandlerFoundForServiceEndpointWhileProcessingMessageExchange"));
                return;
            }
            handler.setDeliveryChannel(channel);
            handler.setMessageExchange(exchange);
            handlerThreadPool.execute(handler);
        } catch (MessagingException e) {
            log.log(Level.WARNING,messages.getString("CJBIRT-W00765.exceptionProcessingMessageExchange",e));
        }
    }

    private AbstractMessageExchangeHandler findHandler(QName serviceName, String endpointName, MessageExchange.Role role) {
        Endpoint endpoint = serviceUnitManager.getEndpoint(serviceName, endpointName, role);
        if (endpoint != null) {
            return endpoint.getProcessor();
        }
        return null;
    }

}
