/**
 *   common-jbi-runtime.jar - Set of classes used in building JBI Components
 *
 *   Copyright (C) 2007 Gestalt, LLC. All Rights Reserved.
 *   http://www.gestalt-llc.com
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License version 2.1 as published by the Free Software Foundation.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
 */
package com.gestalt.jbi.component.manager;

import com.gestalt.jbi.component.AbstractComponent;
import com.gestalt.jbi.component.lifecycle.AbstractComponentLifeCycle;
import com.gestalt.jbi.component.manager.deployment.AbstractDeployer;
import com.sun.jbi.internationalization.Messages;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;
import java.util.logging.Level;

import javax.jbi.component.ComponentContext;
import javax.jbi.component.ServiceUnitManager;
import javax.jbi.management.DeploymentException;
import javax.jbi.messaging.MessageExchange;

import javax.xml.namespace.QName;


/**
 * Abstract implementation of the JBI interface
 * javax.jbi.component.ServiceUnitManager. This interface is optional for
 * JBI components and is only required when the component is a container.
 * A JBI container is similiar to a webapp container or EJB container as it
 * expects to receive Service Units. For example most Binding Components
 * are containers receiving Service Units comprised of a WSDL and jbi.xml.
 * @see javax.jbi.component.ServiceUnitManager
 */
public abstract class AbstractServiceUnitManager implements ServiceUnitManager {
    protected Logger log = Messages.getLogger(AbstractServiceUnitManager.class);
    private Messages messages = Messages.getMessages(AbstractServiceUnitManager.class);
    
    private Map<String, ServiceUnit> serviceUnits =
        new HashMap<String, ServiceUnit>();
    private Map<String, Endpoint> endpoints = new HashMap<String, Endpoint>();
    protected AbstractComponent component;
    protected AbstractDeployer[] deployers;

    public AbstractServiceUnitManager(AbstractComponent component,
        AbstractDeployer[] deployers) {
        this.component = component;
        this.deployers = deployers;
    }

    public void init(String serviceUnitName, String serviceUnitRootPath)
            throws DeploymentException {
        log.log(Level.INFO, messages.getString("CJBIRT-R00815.initializingServiceUnit",serviceUnitName));
        ServiceUnit serviceUnit = serviceUnits.get(serviceUnitName);

        if (serviceUnit == null) {
            deploy(serviceUnitName, serviceUnitRootPath);
        }

        try {
            doInit(serviceUnitName, serviceUnitRootPath);
        } catch (Exception e) {
            log.log(Level.WARNING,messages.getString("CJBIRT-W00817.unableToInitializeTheServiceUnit"));
            throw new DeploymentException("Failed to init the service unit - " +
                serviceUnitName + " : " + e.getMessage(), e);
        }
        log.log(Level.INFO, messages.getString("CJBIRT-R00820.serviceUnitInitialized", serviceUnitName));
    }

    public void start(String serviceUnitName) throws DeploymentException {
        log.log(Level.INFO, messages.getString("CJBIRT-R00822.startingServiceUnit",serviceUnitName));
        ServiceUnit serviceUnit = serviceUnits.get(serviceUnitName);

        if (serviceUnit == null) {
            log.log(Level.WARNING,messages.getString("CJBIRT-W00825.unableToStartTheServiceUnitServiceUnitIsNull"));
            throw new DeploymentException(
                "A null service unit was found for - " + serviceUnitName);
        }

        if (!ServiceUnit.LifeCycle.STOPPED.equals(serviceUnit.getStatus()) &&
                    !ServiceUnit.LifeCycle.SHUTDOWN.equals(
                        serviceUnit.getStatus())) {
            log.log(Level.WARNING,messages.getString("CJBIRT-W00827.serviceUnitMustBeShutdownOrStoppedBeforeItCanBeStarted"));
            throw new DeploymentException(
                "Failed to start the service unit since it was not shutdown or stopped - " +
                serviceUnitName);
        }

        try {
            serviceUnit.start();
        } catch (Exception e) {
            throw new DeploymentException("Failed to start service unit - " +
                serviceUnitName + " : " + e.getMessage(), e);
        }
        log.log(Level.INFO, messages.getString("CJBIRT-R00830.serviceUnitStarted",serviceUnitName));
    }

    public String deploy(String serviceUnitName, String serviceUnitRootPath)
            throws DeploymentException {
        log.log(Level.INFO, messages.getString("CJBIRT-R00832.deployingServiceUnit", serviceUnitName));
        ServiceUnit serviceUnit = serviceUnits.get(serviceUnitName);

        if (serviceUnit != null) {
            log.warning(messages.getString("CJBIRT-W00835.serviceunitWasFoundFor", serviceUnitName));
            throw new DeploymentException("A service unit was found for - " +
                serviceUnitName);
        }

        try {
            serviceUnit = doDeploy(serviceUnitName, serviceUnitRootPath);
        } catch (Exception e) {
            throw new DeploymentException("Failed to deploy the service unit - " +
                serviceUnitName + " : " + e.getMessage(), e);
        }

        if (serviceUnit == null) {
            log.warning(messages.getString("CJBIRT-W00837.nullServiceUnitWasFoundFor",
                serviceUnitName));
            throw new DeploymentException(
                "A null service unit was found for - " + serviceUnitName);
        }

        serviceUnits.put(serviceUnitName, serviceUnit);

        ComponentContext componentContext =
            ((AbstractComponentLifeCycle) component.getLifeCycle()).getComponentContext();

        log.log(Level.INFO, messages.getString("CJBIRT-R00840.serviceUnitDeployed", serviceUnitName));

        return createComponentTaskResultXML(componentContext.getComponentName(),
            "Service unit deployed", true);
    }

    private ServiceUnit doDeploy(String serviceUnitName,
        String serviceUnitRootPath) throws Exception {
        for (AbstractDeployer deployer : deployers) {
            if (deployer.canDeploy(serviceUnitRootPath)) {
                if (log.isLoggable(Level.FINE)) {
                    log.log(Level.FINE,"Deploying the service unit - " + serviceUnitName);
                }
                return deployer.deploy(serviceUnitName, serviceUnitRootPath);
            }
        }

        log.warning(messages.getString("CJBIRT-W00842.noDeployerWasFoundFor", serviceUnitName));

        return null;
    }

    /**
     * Stop this Service Unit. Will indirectly call the Endpoints deactivate()
     * method.
     * Currently don't pass exceptions up the stack since Netbeans
     * doesn't properly handle DeploymentExceptions. Take the SIP BC for
     * example, if we undeploy a ServiceUnit with the proxy down, an
     * exception occurs and even if we call Netbeans force Undeploy,
     * it still doesn't undeploy. So for now just log the exception.
     * @param serviceUnitName The name of the ServiceUnit
     * @throws DeploymentException
     */
    public void stop(String serviceUnitName) throws DeploymentException {
        log.log(Level.INFO, messages.getString("CJBIRT-R00845.stoppingServiceUnit", serviceUnitName));
        ServiceUnit serviceUnit = serviceUnits.get(serviceUnitName);

        if (serviceUnit == null) {
            log.log(Level.WARNING,messages.getString("CJBIRT-W00847.unableToStopTheServiceUnitServiceUnitIsNull"));
            throw new DeploymentException(
                "A null service unit was found for - " + serviceUnitName);
        }

        if (!ServiceUnit.LifeCycle.STARTED.equals(serviceUnit.getStatus())) {
            log.log(Level.WARNING,messages.getString("CJBIRT-W00850.unableToStopAServiceUnitThatIsNotStarted"));
            throw new DeploymentException(
                "Failed to stop the service unit since it was not started - " +
                serviceUnitName);
        }

        try {
            serviceUnit.stop();
        } catch (Exception e) {
            throw new DeploymentException("Failed to stop the service unit - " +
                    serviceUnitName + " : " + e.getMessage());
        }
        log.log(Level.INFO, messages.getString("CJBIRT-R00852.serviceUnitStopped", serviceUnitName));
    }

    public void shutDown(String serviceUnitName) throws DeploymentException {
        log.log(Level.INFO, messages.getString("CJBIRT-R00855.shuttingServiceUnitDown", serviceUnitName));
        ServiceUnit serviceUnit = serviceUnits.get(serviceUnitName);

        if (serviceUnit == null) {
            log.log(Level.WARNING,messages.getString("CJBIRT-W00857.unableToShutdownTheServiceUnitServiceUnitIsNull"));
            throw new DeploymentException(
                "A null service unit was found for - " + serviceUnitName);
        }

        if (!ServiceUnit.LifeCycle.STOPPED.equals(serviceUnit.getStatus())) {
            log.log(Level.WARNING,messages.getString("CJBIRT-W00860.unableToShutdownAServiceUnitThatIsNotStopped"));
            throw new DeploymentException(
                "Failed to shutdown the service unit since it was not stopped - " +
                serviceUnitName);
        }

        try {
            serviceUnit.shutdown();
        } catch (Exception e) {
            throw new DeploymentException("Failed to shutdown the service unit - " +
                serviceUnitName + " : " + e.getMessage(), e);
        }

        log.log(Level.INFO, messages.getString("CJBIRT-R00862.serviceUnitShutdown", serviceUnitName));
    }

    public String undeploy(String serviceUnitName, String serviceUnitRootPath)
            throws DeploymentException {
        log.log(Level.INFO, messages.getString("CJBIRT-R00865.undeployingServiceUnit", serviceUnitName));
        ServiceUnit serviceUnit = serviceUnits.get(serviceUnitName);

        if (serviceUnit == null) {
            log.log(Level.WARNING,messages.getString("CJBIRT-W00867.unableToUndeployTheServiceUnitServiceUnitIsNull"));
            throw new DeploymentException(
                "A null service unit was found for - " + serviceUnitName);
        }

        if (!ServiceUnit.LifeCycle.SHUTDOWN.equals(serviceUnit.getStatus())) {
            log.log(Level.WARNING,messages.getString("CJBIRT-W00870.unableToUndeployAServiceUnitThatIsNotShutdown"));
            throw new DeploymentException(
                "Failed to undeploy the service unit since it was not shutdown - " +
                serviceUnitName);
        }

        try {
            doUndeploy(serviceUnitName, serviceUnitRootPath, serviceUnit);
            serviceUnits.remove(serviceUnitName);
        } catch (Exception e) {
            throw new DeploymentException("Failed to undeploy the service unit - " +
                serviceUnitName + " : " + e.getMessage(), e);
        }

        ComponentContext componentContext =
            ((AbstractComponentLifeCycle) component.getLifeCycle()).getComponentContext();

        log.log(Level.INFO, messages.getString("CJBIRT-R00872.undeployedServiceUnit", serviceUnitName));

        return createComponentTaskResultXML(componentContext.getComponentName(),
            "Service unit undeployed", true);
    }

    private void doUndeploy(String serviceUnitName, String serviceUnitRootPath,
        ServiceUnit serviceUnit) throws Exception {
        for (AbstractDeployer deployer : deployers) {
            if (deployer.canDeploy(serviceUnitRootPath)) {
                if (log.isLoggable(Level.FINE)) {
                    log.log(Level.FINE,"Undeploying the service unit - " + serviceUnitName);
                }
                deployer.shutdown(serviceUnit);
                return;
            }
        }

        log.log(Level.WARNING,messages.getString("CJBIRT-W00875.noDeployerWasFoundFor", serviceUnitName));
    }

    public void registerEndpoint(String endpointKey, Endpoint endpoint) {
        endpoints.put(endpointKey, endpoint);
    }

    public Endpoint getEndpoint(QName serviceName, String endpointName, MessageExchange.Role role) {
        return endpoints.get(ServiceUnit.generateEndpointKey(serviceName,
                endpointName, role));
    }

    public Collection<ServiceUnit> getServiceUnits() {
        return serviceUnits.values();
    }

    /**
     * helper method to create result message in management message xml.
     * @param componentName The Name of the Component
     * @param message message string to return
     * @param isSuccess true to format a sucess result, false to
     * format a failed result.
     * @return XML string.
     */
    protected String createComponentTaskResultXML(String componentName,
        String message, boolean isSuccess) {
        String taskResult = isSuccess ? "SUCCESS" : "FAILED";
        String msgType = isSuccess ? "INFO" : "ERROR";

        return "<component-task-result xmlns=\"http://java.sun.com/xml/ns/jbi/management-message\" >" +
        "  <component-name>" + componentName + "</component-name>" +
        "  <component-task-result-details >" + "      <task-result-details>" +
        "          <task-id>deployment</task-id>" + "          <task-result>" +
        taskResult + "</task-result>" + "          <message-type>" + msgType +
        "</message-type>" + "          <task-status-msg>" +
        "             <msg-loc-info>" +
        "                <loc-token>MSG_ID_000</loc-token>" +
        "                <loc-message>" + message + "</loc-message>" +
        "              </msg-loc-info>" + "          </task-status-msg>" +
        "      </task-result-details>" + "  </component-task-result-details>" +
        "</component-task-result>";
    }

    public abstract void doInit(String serviceUnitName,
        String serviceUnitRootPath) throws Exception;
}
