/**
 *   common-jbi-runtime.jar - Set of classes used in building JBI Components
 *
 *   Copyright (C) 2007 Gestalt, LLC. All Rights Reserved.
 *   http://www.gestalt-llc.com/
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License version 2.1 as published by the Free Software Foundation.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
 */
package com.gestalt.jbi.component.manager.deployment;

import com.gestalt.jbi.component.AbstractComponent;
import com.gestalt.jbi.component.descriptor.JBIDescriptorParser;
import com.gestalt.jbi.component.manager.ServiceUnit;

import com.sun.jbi.internationalization.Messages;

import java.io.File;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.management.DeploymentException;
import javax.jbi.messaging.MessageExchange;
import static javax.jbi.messaging.MessageExchange.Role.CONSUMER;
import static javax.jbi.messaging.MessageExchange.Role.PROVIDER;

import javax.xml.namespace.QName;


public abstract class AbstractDeployer {
    protected Logger log = Messages.getLogger(AbstractDeployer.class);
    private Messages messages = Messages.getMessages(AbstractDeployer.class);
    protected AbstractComponent component;

    public AbstractDeployer(AbstractComponent component) {
        this.component = component;
    }

    public void shutdown(ServiceUnit serviceUnit) throws DeploymentException {
        log.log(Level.INFO,
            messages.getString("CJBIRT-R00800.serviceUnit",
                serviceUnit.getServiceUnitName()));

        try {
            serviceUnit.shutdown();
        } catch (Exception e) {
            log.log(Level.WARNING,
                messages.getString(
                    "CJBIRT-W00805.unableToShutdownTheServiceUnit"));
            throw new DeploymentException(
                "Failed to shutdown the service unit - " +
                serviceUnit.getServiceUnitName() + " : " + e.getMessage(), e);
        }
    }

    public boolean canDeploy(String serviceUnitRootPath) {
        List<File> wsdls = scanDirectory(new File(serviceUnitRootPath));
        boolean canDeploy = ((wsdls == null) ? 0 : wsdls.size()) > 0;

        if (log.isLoggable(Level.FINEST)) {
            log.log(Level.FINEST, "We canDeploy was found to be - " +
                canDeploy);
        }

        return canDeploy;
    }

    protected List<File> scanDirectory(File directory) {
        File[] files = directory.listFiles();
        List<File> tmp = new ArrayList<File>();

        for (File file : files) {
            if (file.isDirectory()) {
                tmp.addAll(scanDirectory(file));
            } else {
                if (isDeployable(file)) {
                    tmp.add(file);
                }
            }
        }

        return tmp;
    }

    protected abstract boolean isDeployable(File file);

    public ServiceUnit deploy(String serviceUnitName, String serviceUnitRootPath)
        throws DeploymentException {
        log.log(Level.INFO,
            messages.getString("CJBIRT-R00701.serviceUnitDeploying"));

        List<File> wsdls = scanDirectory(new File(serviceUnitRootPath));

        if (((wsdls == null) ? 0 : wsdls.size()) == 0) {
            log.warning(messages.getString(
                    "CJBIRT-W00955.failedToDeployServiceUnitNoFilesFound",
                    serviceUnitName));
            throw new DeploymentException(
                "Failed to deploy the service unit - " +
                "no deployable files were found - " + serviceUnitName);
        }

        ServiceUnit serviceUnit = createServiceUnit(serviceUnitName,
                serviceUnitRootPath);
        JBIDescriptorParser descriptorParser = new JBIDescriptorParser(serviceUnitRootPath);

        for (File wsdl : wsdls) {
            try {
                if (log.isLoggable(Level.FINER)) {
                    log.log(Level.FINER,
                        "Trying to parse file - " + wsdl.getAbsolutePath());
                }

                parse(serviceUnit, wsdl, descriptorParser);
            } catch (Exception e) {
                log.log(Level.WARNING, "Cannot parse the file", e);
            }
        }

        if (serviceUnit.getEndpoints().size() == 0) {
            log.warning(messages.getString(
                    "CJBIRT-W00960.failedToDeployServiceUnitNoEndpointsFound  ",
                    serviceUnitName));
            throw new DeploymentException(
                "Failed to deploy the service unit - " +
                "no endpoints were found - " + serviceUnitName);
        }

        log.log(Level.INFO,
            messages.getString("CJBIRT-W00965.serviceUnitDeployed"));

        return serviceUnit;
    }

    protected ServiceUnit createServiceUnit(String serviceUnitName,
        String serviceUnitRootPath) {
        return new ServiceUnit(serviceUnitName, serviceUnitRootPath, component);
    }

    /**
     * Handles the parsing of the WSDL and will create the endpoints for
     * providers and consumers,
     */
    protected void parse(ServiceUnit serviceUnit, File file,
        JBIDescriptorParser descriptorParser) throws Exception {
        if (log.isLoggable(Level.FINE)) {
            log.log(Level.FINE, "Parsing file: " + file);
        }

        for (JBIDescriptorParser.Consumer consumer : descriptorParser.getConsumers()) {
            addEndpoint(serviceUnit, file, consumer.getServiceName(),
                consumer.getEndpointName(), consumer.getInterfaceName(),
                CONSUMER);
        }

        for (JBIDescriptorParser.Provider provider : descriptorParser.getProviders()) {
            addEndpoint(serviceUnit, file, provider.getServiceName(),
                provider.getEndpointName(), provider.getInterfaceName(),
                PROVIDER);
        }

        if (log.isLoggable(Level.FINE)) {
            log.fine("Processing Complete");
        }
    }

    protected abstract void addEndpoint(ServiceUnit serviceUnit, File file,
        QName serviceName, String endpointName, QName interfaceName,
        MessageExchange.Role role) throws Exception;
}
